@@ -33,7 +33,6 @@
 #define PST_TYPE_OTHER      13
 #define PST_TYPE_REPORT     14
 
-
 // defines types of possible encryption
 #define PST_NO_ENCRYPT   0
 #define PST_COMP_ENCRYPT 1
@@ -72,6 +71,27 @@
 #define PST_APP_RECUR_MONTHLY     3
 #define PST_APP_RECUR_YEARLY      4
 
+// define attachment types
+#define PST_ATTACH_NONE             0
+#define PST_ATTACH_BY_VALUE         1
+#define PST_ATTACH_BY_REF           2
+#define PST_ATTACH_BY_REF_RESOLV    3
+#define PST_ATTACH_BY_REF_ONLY      4
+#define PST_ATTACH_EMBEDDED         5
+#define PST_ATTACH_OLE              6
+
+// define flags
+#define PST_FLAG_READ			0x01
+#define PST_FLAG_UNMODIFIED		0x02
+#define PST_FLAG_SUBMIT			0x04
+#define PST_FLAG_UNSENT			0x08
+#define PST_FLAG_HAS_ATTACHMENT	0x10
+#define PST_FLAG_FROM_ME		0x20
+#define PST_FLAG_ASSOCIATED		0x40
+#define PST_FLAG_RESEND			0x80
+#define PST_FLAG_RN_PENDING		0x100
+#define PST_FLAG_NRN_PENDING	0x200
+
 
 typedef struct pst_entryid {
     int32_t u1;
@@ -299,6 +319,16 @@ typedef struct pst_item_email {
     pst_string  supplementary_info;
     /** mapi element 0x0c20 PR_NDR_STATUS_CODE */
     int32_t     ndr_status_code;
+
+    // elements added for .msg processing
+    /** mapi element 0x0040 PR_RECEIVED_BY_NAME */
+    pst_string  outlook_received_name1;
+    /** mapi element 0x0c1a PR_SENDER_NAME */
+    pst_string  outlook_sender_name2;
+    /** mapi element 0x0e1d PR_NORMALIZED_SUBJECT */
+    pst_string  outlook_normalized_subject;
+    /** mapi element 0x300b PR_SEARCH_KEY */
+    pst_string  outlook_search_key;
 } pst_item_email;
 
 
@@ -740,6 +770,10 @@ typedef struct pst_item_appointment {
  *  each major mapi item type. It represents a complete mapi object.
  */
 typedef struct pst_item {
+    /** pointer to the pst_file */
+    struct pst_file        *pf;
+    /** block id that can be used to generate uid */
+    uint64_t               block_id;
     /** email mapi elements */
     pst_item_email         *email;
     /** folder mapi elements */
@@ -856,6 +890,8 @@ typedef struct pst_file {
     char*   cwd;
     /** original file name when the file was opened */
     char*   fname;
+    /** default character set for items without one */
+    char*   charset;
     /** the head and tail of the linked list of index structures */
     pst_index_ll *i_head, *i_tail;
     /** the head and tail of the top level of the descriptor tree */
@@ -894,12 +930,13 @@ typedef struct pst_file {
 
 
 /** Open a pst file.
- * @param pf   pointer to uninitialized pst_file structure. This structure
- *             will be filled in by this function.
- * @param name name of the file, suitable for fopen().
+ * @param pf       pointer to uninitialized pst_file structure. This structure
+ *                 will be filled in by this function.
+ * @param name     name of the file, suitable for fopen().
+ * @param charset  default charset for item with unspecified character sets
  * @return 0 if ok, -1 if error
  */
-int             pst_open(pst_file *pf, const char *name);
+int             pst_open(pst_file *pf, const char *name, const char *charset);
 
 
 /** Reopen the pst file after a fork
@@ -1062,6 +1099,20 @@ char*           pst_rfc2445_datetime_format_now(int buflen, char* result);
 const char*     pst_default_charset(pst_item *item, int buflen, char* result);
 
 
+/** Convert str to rfc2231 encoding of str
+ *  @param str   pointer to the mapi string of interest
+ */
+void            pst_rfc2231(pst_string *str);
+
+
+/** Convert str to rfc2047 encoding of str, possibly enclosed in quotes if it contains spaces
+ *  @param item          pointer to the containing mapi item
+ *  @param str           pointer to the mapi string of interest
+ *  @param needs_quote   true if strings containing spaces should be wrapped in quotes
+ */
+void            pst_rfc2047(pst_item *item, pst_string *str, int needs_quote);
+
+
 /** Convert str to utf8 if possible; null strings are preserved.
  * @param item  pointer to the containing mapi item
  * @param str   pointer to the mapi string of interest
