@@ -167,9 +167,6 @@ void version() {
 	printf("Little Endian implementation being used.\n");
 #else
 #  error "Byte order not supported by this library"
-#endif
-#ifdef __GNUC__
-			 printf("GCC %d.%d : %s %s\n", __GNUC__, __GNUC_MINOR__, __DATE__, __TIME__);
 #endif
 	 DEBUG_RET();
 }
@@ -215,7 +212,7 @@ int main(int argc, char* const* argv) {
 	}
 
     // Open PST file
-    if (pst_open(&pstfile, argv[optind])) DIE(("Error opening File\n"));
+    if (pst_open(&pstfile, argv[optind], NULL)) DIE(("Error opening File\n"));
 
     // Load PST index
     if (pst_load_index(&pstfile)) DIE(("Index Error\n"));
