@@ -7,6 +7,7 @@
 
 #include "define.h"
 #include "lzfu.h"
+#include "msg.h"
 
 #define OUTPUT_TEMPLATE "%s"
 #define OUTPUT_KMAIL_DIR_TEMPLATE ".%s.directory"
@@ -39,7 +40,7 @@ char*     mk_recurse_dir(char* dir, int32_t folder_type);
 int       close_recurse_dir();
 char*     mk_separate_dir(char *dir);
 int       close_separate_dir();
-void      mk_separate_file(struct file_ll *f, char *extension);
+void      mk_separate_file(struct file_ll *f, char *extension, int openit);
 void      close_separate_file(struct file_ll *f);
 char*     my_stristr(char *haystack, char *needle);
 void      check_filename(char *fname);
@@ -58,7 +59,7 @@ void      find_rfc822_headers(char** extra_mime_headers);
 void      write_body_part(FILE* f_output, pst_string *body, char *mime, char *charset, char *boundary, pst_file* pst);
 void      write_schedule_part_data(FILE* f_output, pst_item* item, const char* sender, const char* method);
 void      write_schedule_part(FILE* f_output, pst_item* item, const char* sender, const char* boundary);
-void      write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode, int mode_MH, pst_file* pst, int save_rtf, char** extra_mime_headers);
+void      write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode, int mode_MH, pst_file* pst, int save_rtf, int embedding, char** extra_mime_headers);
 void      write_vcard(FILE* f_output, pst_item *item, pst_item_contact* contact, char comment[]);
 int       write_extra_categories(FILE* f_output, pst_item* item);
 void      write_journal(FILE* f_output, pst_item* item);
@@ -122,6 +123,7 @@ char*  kmail_chdir = NULL;
 int         mode         = MODE_NORMAL;
 int         mode_MH      = 0;   // a submode of MODE_SEPARATE
 int         mode_EX      = 0;   // a submode of MODE_SEPARATE
+int         mode_MSG     = 0;   // a submode of MODE_SEPARATE
 int         mode_thunder = 0;   // a submode of MODE_RECURSE
 int         output_mode  = OUTPUT_NORMAL;
 int         contact_mode = CMODE_VCARD;
@@ -306,7 +308,7 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                 }
                 else {
                     ff.item_count++;
-                    if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".vcf" : "");
+                    if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".vcf" : "", 1);
                     if (contact_mode == CMODE_VCARD) {
                         pst_convert_utf8_null(item, &item->comment);
                         write_vcard(ff.output, item, item->contact, item->comment.str);
@@ -342,9 +344,13 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                         if (child == 0) {
                             // we are the child process, or the original parent if no children were available
                             pid_t me = getpid();
-                            mk_separate_file(&ff, (mode_EX) ? ".eml" : "");
-                            write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, &extra_mime_headers);
+                            mk_separate_file(&ff, (mode_EX) ? ".eml" : "", 1);
+                            write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, 0, &extra_mime_headers);
                             close_separate_file(&ff);
+                            if (mode_MSG) {
+                                mk_separate_file(&ff, ".msg", 0);
+                                write_msg_email(ff.name, item, &pstfile);
+                            }
 #ifdef HAVE_FORK
 #ifdef HAVE_SEMAPHORE_H
                             if (me != parent) {
@@ -361,7 +367,7 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                     }
                     else {
                         // process this single email message, cannot fork since not separate mode
-                        write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, &extra_mime_headers);
+                        write_normal_email(ff.output, ff.name, item, mode, mode_MH, &pstfile, save_rtf_body, 0, &extra_mime_headers);
                     }
                 }
             }
@@ -380,7 +386,7 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                 }
                 else {
                     ff.item_count++;
-                    if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".ics" : "");
+                    if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".ics" : "", 1);
                     write_journal(ff.output, item);
                     fprintf(ff.output, "\n");
                     if (mode == MODE_SEPARATE) close_separate_file(&ff);
@@ -401,7 +407,7 @@ void process(pst_item *outeritem, pst_desc_tree *d_ptr)
                 }
                 else {
                     ff.item_count++;
-                    if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".ics" : "");
+                    if (mode == MODE_SEPARATE) mk_separate_file(&ff, (mode_EX) ? ".ics" : "", 1);
                     write_schedule_part_data(ff.output, item, NULL, NULL);
                     fprintf(ff.output, "\n");
                     if (mode == MODE_SEPARATE) close_separate_file(&ff);
@@ -444,7 +450,7 @@ int main(int argc, char* const* argv) {
     }
 
     // command-line option handling
-    while ((c = getopt(argc, argv, "bC:c:Dd:ehj:kMo:qrSt:uVw"))!= -1) {
+    while ((c = getopt(argc, argv, "bC:c:Dd:emhj:kMo:qrSt:uVw"))!= -1) {
         switch (c) {
         case 'b':
             save_rtf_body = 0;
@@ -491,13 +497,22 @@ int main(int argc, char* const* argv) {
             break;
         case 'M':
             mode = MODE_SEPARATE;
-            mode_MH = 1;
-            mode_EX = 0;
+            mode_MH  = 1;
+            mode_EX  = 0;
+            mode_MSG = 0;
             break;
         case 'e':
             mode = MODE_SEPARATE;
-            mode_MH = 1;
-            mode_EX = 1;
+            mode_MH  = 1;
+            mode_EX  = 1;
+            mode_MSG = 0;
+            file_name_len = 14;
+            break;
+        case 'm':
+            mode = MODE_SEPARATE;
+            mode_MH  = 1;
+            mode_EX  = 1;
+            mode_MSG = 1;
             file_name_len = 14;
             break;
         case 'o':
@@ -512,8 +527,9 @@ int main(int argc, char* const* argv) {
             break;
         case 'S':
             mode = MODE_SEPARATE;
-            mode_MH = 0;
-            mode_EX = 0;
+            mode_MH  = 0;
+            mode_EX  = 0;
+            mode_MSG = 0;
             break;
         case 't':
             // email, appointment, contact, other
@@ -708,7 +724,7 @@ void usage() {
     printf("Usage: %s [OPTIONS] {PST FILENAME}\n", prog_name);
     printf("OPTIONS:\n");
     printf("\t-V\t- Version. Display program version\n");
-    printf("\t-C charset\t- character set for items with unspecified character set\n");
+    printf("\t-C charset\t- character set for items with an unspecified character set\n");
     printf("\t-D\t- Include deleted items in output\n");
     printf("\t-M\t- Write emails in the MH (rfc822) format\n");
     printf("\t-S\t- Separate. Write emails in the separate format\n");
@@ -719,6 +735,7 @@ void usage() {
     printf("\t-h\t- Help. This screen\n");
     printf("\t-j <integer>\t- Number of parallel jobs to run\n");
     printf("\t-k\t- KMail. Output in kmail format\n");
+    printf("\t-m\t- As with -e, but write .msg files also\n");
     printf("\t-o <dirname>\t- Output directory to write files to. CWD is changed *after* opening pst file\n");
     printf("\t-q\t- Quiet. Only print error messages\n");
     printf("\t-r\t- Recursive. Output in a recursive format\n");
@@ -726,7 +743,7 @@ void usage() {
     printf("\t-u\t- Thunderbird mode. Write two extra .size and .type files\n");
     printf("\t-w\t- Overwrite any output mbox files\n");
     printf("\n");
-    printf("Only one of -k -M -r -S should be specified\n");
+    printf("Only one of -M -S -e -k -m -r should be specified\n");
     DEBUG_RET();
 }
 
@@ -924,7 +941,7 @@ int close_separate_dir() {
 }
 
 
-void mk_separate_file(struct file_ll *f, char *extension) {
+void mk_separate_file(struct file_ll *f, char *extension, int openit) {
     DEBUG_ENT("mk_separate_file");
     DEBUG_INFO(("opening next file to save email\n"));
     if (f->item_count > 999999999) { // bigger than nine 9's
@@ -932,8 +949,10 @@ void mk_separate_file(struct file_ll *f, char *extension) {
     }
     sprintf(f->name, SEP_MAIL_FILE_TEMPLATE, f->item_count, extension);
     check_filename(f->name);
-    if (!(f->output = fopen(f->name, "w"))) {
-        DIE(("mk_separate_file: Cannot open file to save email \"%s\"\n", f->name));
+    if (openit) {
+        if (!(f->output = fopen(f->name, "w"))) {
+            DIE(("mk_separate_file: Cannot open file to save email \"%s\"\n", f->name));
+        }
     }
     DEBUG_RET();
 }
@@ -1082,7 +1101,7 @@ void write_embedded_message(FILE* f_output, pst_item_attach* attach, char *bound
         } else {
             fprintf(f_output, "\n--%s\n", boundary);
             fprintf(f_output, "Content-Type: %s\n\n", attach->mimetype.str);
-            write_normal_email(f_output, "", item, MODE_NORMAL, 0, pf, save_rtf, extra_mime_headers);
+            write_normal_email(f_output, "", item, MODE_NORMAL, 0, pf, save_rtf, 1, extra_mime_headers);
         }
         pst_freeItem(item);
     }
@@ -1305,7 +1324,7 @@ void find_rfc822_headers(char** extra_mime_headers)
         char *temp, *t;
         while ((temp = strstr(headers, "\n\n"))) {
             temp[1] = '\0';
-            t = header_get_field(headers, "\nContent-Type: ");
+            t = header_get_field(headers, "\nContent-Type:");
             if (t) {
                 t++;
                 DEBUG_INFO(("found content type header\n"));
@@ -1420,7 +1439,7 @@ void write_schedule_part(FILE* f_output, pst_item* item, const char* sender, con
 }
 
 
-void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode, int mode_MH, pst_file* pst, int save_rtf, char** extra_mime_headers)
+void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode, int mode_MH, pst_file* pst, int save_rtf, int embedding, char** extra_mime_headers)
 {
     char boundary[60];
     char altboundary[66];
@@ -1493,21 +1512,21 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
         }
 
         // Check if the headers have all the necessary fields
-        header_has_field(headers, "\nFrom: ",        &has_from);
-        header_has_field(headers, "\nTo: ",          &has_to);
-        header_has_field(headers, "\nSubject: ",     &has_subject);
-        header_has_field(headers, "\nDate: ",        &has_date);
-        header_has_field(headers, "\nCC: ",          &has_cc);
-        header_has_field(headers, "\nMessage-Id: ",  &has_msgid);
+        header_has_field(headers, "\nFrom:",        &has_from);
+        header_has_field(headers, "\nTo:",          &has_to);
+        header_has_field(headers, "\nSubject:",     &has_subject);
+        header_has_field(headers, "\nDate:",        &has_date);
+        header_has_field(headers, "\nCC:",          &has_cc);
+        header_has_field(headers, "\nMessage-Id:",  &has_msgid);
 
         // look for charset and report-type in Content-Type header
-        t = header_get_field(headers, "\nContent-Type: ");
+        t = header_get_field(headers, "\nContent-Type:");
         header_get_subfield(t, "charset", body_charset, sizeof(body_charset));
         header_get_subfield(t, "report-type", body_report, sizeof(body_report));
 
         // derive a proper sender email address
         if (!sender_known) {
-            t = header_get_field(headers, "\nFrom: ");
+            t = header_get_field(headers, "\nFrom:");
             if (t) {
                 // assume address is on the first line, rather than on a continuation line
                 t++;
@@ -1525,13 +1544,12 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
 
         // Strip out the mime headers and some others that we don't want to emit
         header_strip_field(headers, "\nMicrosoft Mail Internet Headers");
-        header_strip_field(headers, "\nMIME-Version: ");
-        header_strip_field(headers, "\nContent-Type: ");
-        header_strip_field(headers, "\nContent-Transfer-Encoding: ");
-        header_strip_field(headers, "\nContent-class: ");
-        header_strip_field(headers, "\nX-MimeOLE: ");
-        header_strip_field(headers, "\nBcc:");
-        header_strip_field(headers, "\nX-From_: ");
+        header_strip_field(headers, "\nMIME-Version:");
+        header_strip_field(headers, "\nContent-Type:");
+        header_strip_field(headers, "\nContent-Transfer-Encoding:");
+        header_strip_field(headers, "\nContent-class:");
+        header_strip_field(headers, "\nX-MimeOLE:");
+        header_strip_field(headers, "\nX-From_:");
     }
 
     DEBUG_INFO(("About to print Header\n"));
@@ -1546,7 +1564,8 @@ void write_normal_email(FILE* f_output, char f_name[], pst_item* item, int mode,
         // procmail produces this separator without the quotes around the
         // sender email address, but apparently some Mac email client needs
         // those quotes, and they don't seem to cause problems for anyone else.
-        fprintf(f_output, "From \"%s\" %s\n", sender, c_time);
+        char *quo = (embedding) ? ">" : "";
+        fprintf(f_output, "%sFrom \"%s\" %s\n", quo, sender, c_time);
     }
 
     // print the supplied email headers
